body
{
  background-color: #ffffff;
  color: #000000;
}

img
{
  display: block;
  margin-left: auto;
  margin-right: auto
}

a:link
{
  text-decoration: none;
  color: blue;
  font-size: 12pt;
  font-weight: bold;
}
