# if defined(UNDER_CE)
#  include <winbase.h>
# else
#  include <winver.h>
# endif

SPEEDCRUNCH_ICON ICON DISCARDABLE "speedcrunch.ico"

VS_VERSION_INFO VERSIONINFO
	FILEVERSION 0,12,0,0
	PRODUCTVERSION 0,12,0,0
	FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
	FILEFLAGS VS_FF_DEBUG
#else
	FILEFLAGS 0x0L
#endif
	FILEOS VOS__WINDOWS32
	FILETYPE VFT_APP
	FILESUBTYPE 0x0L
	BEGIN
		BLOCK "StringFileInfo"
		BEGIN
			BLOCK "040904B0"
			BEGIN
				VALUE "CompanyName", "http://SpeedCrunch.org\0"
				VALUE "FileVersion", "trunk\0"
				VALUE "LegalCopyright", "Copyright 2004-2009 http://SpeedCrunch.org\0"
			END
		END
	END
/* End of Version info */
